'use strict';

exports.__esModule = true;

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

exports.parseCSS = parseCSS;
exports.expandCSSFallbacks = expandCSSFallbacks;

var _parse = require('styled-components/lib/vendor/postcss-safe-parser/parse');

var _parse2 = _interopRequireDefault(_parse);

var _postcssNested = require('styled-components/lib/vendor/postcss-nested');

var _postcssNested2 = _interopRequireDefault(_postcssNested);

var _stringify = require('styled-components/lib/vendor/postcss/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

var _postcssJs = require('postcss-js');

var _postcssJs2 = _interopRequireDefault(_postcssJs);

var _parser = require('postcss-js/parser');

var _parser2 = _interopRequireDefault(_parser);

var _autoprefixer = require('autoprefixer');

var _autoprefixer2 = _interopRequireDefault(_autoprefixer);

var _CSSPropertyOperations = require('./glamor/CSSPropertyOperations');

var _index = require('./index');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var prefixer = _postcssJs2.default.sync([_autoprefixer2.default]);

function parseCSS(css, extractStatic) {
  // todo - handle errors
  var root = void 0;
  if ((typeof css === 'undefined' ? 'undefined' : _typeof(css)) === 'object') {
    root = (0, _parser2.default)(css);
  } else {
    root = (0, _parse2.default)(css);
  }
  var vars = 0;
  var composes = 0;

  (0, _postcssNested2.default)(root);

  root.walkDecls(function (decl) {
    if (decl.prop === 'composes') {
      if (!/xxx(\d+)xxx/gm.exec(decl.value)) {
        throw new Error('composes must be a interpolation');
      }
      if (decl.parent.nodes[0] !== decl) {
        throw new Error('composes must be the first rule');
      }
      var composeMatches = decl.value.match(/xxx(\d+)xxx/gm);
      var numOfComposes = !composeMatches ? 0 : composeMatches.length;
      composes += numOfComposes;
      vars += numOfComposes;
      decl.remove();
    }
  });

  var styles = expandCSSFallbacks(prefixer(_postcssJs2.default.objectify(root)));

  return {
    styles: styles,
    staticCSSRules: vars === 0 && extractStatic ? stringifyCSSRoot(_postcssJs2.default.parse(styles)) : [],
    composesCount: composes
  };
}

function stringifyCSSRoot(root) {
  return root.nodes.map(function (node, i) {
    var str = '';
    (0, _stringify2.default)(node, function (x) {
      str += x;
    });
    return str;
  });
}

function expandCSSFallbacks(style) {
  var flattened = Object.keys(style).reduce(function (accum, key) {
    if (Array.isArray(style[key])) {
      accum[key] = style[key].join('; ' + (0, _CSSPropertyOperations.processStyleName)(key) + ': ');
    } else if (Object.prototype.toString.call(style[key]) === '[object Object]') {
      accum[key] = expandCSSFallbacks(style[key]);
    } else {
      accum[key] = style[key];
    }
    return accum;
  }, {});
  // todo -
  // flatten arrays which haven't been flattened yet
  return flattened;
}